unit FrmFilter;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Rtti, System.Classes,
  System.Variants, FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Filter,
  FMX.StdCtrls, FMX.Layouts, FMX.ExtCtrls, FMX.Effects, FMX.Filter.Effects,

  FMX.Surfaces;

type
  TF_Filters = class(TForm)
    ImageViewer1: TImageViewer;
    bnOK: TButton;
    Timer1: TTimer;
    TrackBar1: TTrackBar;
    bnCancel: TButton;
    Label1: TLabel;
    PaperSketchEffect1: TPaperSketchEffect;
    procedure bnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

  TMyBitmap = class (TBitmap)
    procedure SaveToStream(Stream: TStream);
  end;

var
  F_Filters: TF_Filters;

Function ShowBitmapFromStream (ms: TMemoryStream): Boolean; export;

implementation

procedure TMyBitmap.SaveToStream(Stream: TStream);
var
  Surf: TBitmapSurface;
begin
  Surf := TBitmapSurface.Create;
  try
    Surf.Assign(Self);
    TBitmapCodecManager.SaveToStream(Stream, Surf, '.bmp');
  finally
    Surf.Free;
  end;
end;

{$R *.fmx}

function ShowBitmapFromStream (ms: TMemoryStream): Boolean;
var
  Filter: FMX.Filter.TFilter;
begin
  Filter := TFilterManager.FilterByName('PaperSketch');

  try
    F_Filters := TF_Filters.Create(Application);
    F_Filters.ImageViewer1.bitmap.LoadFromStream(ms);

    if F_Filters.ShowModal = mrOK then begin
      Filter.ValuesAsBitmap['Input'] := F_Filters.ImageViewer1.bitmap;
      Filter.ValuesAsFloat ['BrushSize'] := F_Filters.TrackBar1.Value;
      F_Filters.ImageViewer1.bitmap := TBitmap (Filter.ValuesAsBitmap['Output']);

      ms.Position := 0;
      TMyBitmap (F_Filters.ImageViewer1.bitmap).saveToStream (ms);

      Result := True;
    end else begin
      Result := false;
    end;

  finally
    F_Filters.Free;
  end;
end;


procedure TF_Filters.bnOKClick(Sender: TObject);
begin
  Modalresult := mrOK;
end;

procedure TF_Filters.FormResize(Sender: TObject);
begin
  // Workaround fr derzeitig bestehender Bug im Imageviewer
  Timer1.Enabled := True;
end;

procedure TF_Filters.FormShow(Sender: TObject);
begin
  // Workaround fr derzeitig bestehender Bug im Imageviewer
  F_Filters.ImageViewer1.BitmapScale := 1;
  Timer1.Enabled := True;
end;

procedure TF_Filters.Timer1Timer(Sender: TObject);
begin
  F_Filters.ImageViewer1.BestFit;
  // Die nchsten beiden Aufrufe sind Workarounds fr XE4/XE5, entfernen, wenn Problem gefixt
  F_Filters.ImageViewer1.ScrollTo(F_Filters.ImageViewer1.width,F_Filters.ImageViewer1.height);
  F_Filters.ImageViewer1.RealignContent;
  Timer1.Enabled := false;
end;

procedure TF_Filters.TrackBar1Change(Sender: TObject);
begin
  PaperSketchEffect1.brushSize := TrackBar1.Value;
  Label1.Text := FloatToStr (TrackBar1.Value);
end;

end.
